﻿using AspectCore.DynamicProxy;
using AspectCore.DynamicProxy.Parameters;
using System.Collections;
using HealthMonitor.Core.Common;

namespace HealthMonitor.Core.Pipeline.Aop.Query
{
    public class AssertValidQueryHisValue : AopValueBase
    {
        public override async Task Invoke(CacheInterceptorContext context)
        {
            var methodReturnType = context.AopContext.GetReturnParameter().Type;
            //先判断方法是否有返回值，无就不进行缓存判断
            if (methodReturnType == typeof(void) || methodReturnType == typeof(Task))
            {
                throw new AssertValidException();
            }
            //方法需要存在参数用于生成缓存键
            if (context.AopContext.Parameters.Length < 3)
            {
                throw new AssertValidException();
            }

            var returnType = methodReturnType;
            //判断是否是异步方法
            bool isAsync = context.AopContext.IsAsync();
            if (isAsync)
            {
                //取得异步返回的类型
                returnType = methodReturnType.GenericTypeArguments.FirstOrDefault();
            }

            var entityType = returnType;
            bool isEnumerable = false;
            if (returnType.GetInterface(typeof(IEnumerable).Name) != null)
            {
                entityType = returnType.GenericTypeArguments.FirstOrDefault();
                isEnumerable = true;
            }

            context.EntityType = entityType;
            context.ReturnType = returnType;
            context.IsAsyncMethod = isAsync;
            context.IsReturnEnumerable = isEnumerable;
            context.RequestId = context.AopContext.Parameters.Length > 3 ? context.AopContext.Parameters[3] + "" : "";

            using (var scope = context.Logger.BeginScope(new Dictionary<string, object> { ["RequestId"] = context.RequestId }))
            using (new CustomizeStopWatch($"查询历史aop[{entityType.Name}]", context.Logger))
            {
                await InvokeNextAsync(context);
            }
        }
    }
}
